var width = 0,
	height = 0,
	pageWidth = window.innerWidth,
    pageHeight = window.innerHeight,
    navHeight,
    breakpointMobile = 1024;

var inferno = d3.select('svg#InfernoWeb'),
	infobox = d3.select('#infobox');

var currentChar = new Object(),
	selection,
	selectedChar;

var mobileButton = d3.select('#mobileButton');

function resizeSVG(){
	pageWidth = window.innerWidth;
    pageHeight = window.innerHeight;
    navHeight = d3.select('#navContainer').style('height').substring(0,2);
	d3.select('svg#InfernoWeb')
		.attr('width', pageWidth)
		.attr('height', pageHeight-navHeight);
};

var zoom = d3.behavior.zoom();

function zoomed() {
	d3.select('#zoomContainer').attr('transform',
        'translate(' + zoom.translate() + ')' +
        'scale(' + zoom.scale() + ')'
    );
};

function toggleZoomBehavior(){
	if (pageWidth >= breakpointMobile) {		
		inferno.call(zoom
		     	.scaleExtent([1, 11.5])
				.on('zoom', zoomed));	
	} else {		
		inferno.call(zoom.on('zoom', null));
	};
};

window.onload = function(){
	resizeSVG();
	toggleZoomBehavior();
};

window.onresize = function(){
	resizeSVG();
	toggleZoomBehavior();
};

function interpolateZoom (translate, scale) {
    var self = this;
    return d3.transition().duration(350).tween('zoom', function () {
        var iTranslate = d3.interpolate(zoom.translate(), translate),
            iScale = d3.interpolate(zoom.scale(), scale);
        return function (t) {
            zoom
                .scale(iScale(t))
                .translate(iTranslate(t));
            zoomed();
        };
    });
}

function zoomClick() {
	var svgViewBox = inferno.attr('viewBox'),
		svgViewBoxValues = svgViewBox.split(' '),
		svgViewBoxBeginX = svgViewBoxValues[0],
		svgViewBoxBeginY = svgViewBoxValues[1],
		svgViewBoxEndX = svgViewBoxValues[2],
		svgViewBoxEndy = svgViewBoxValues[3],
		svgCenterX = svgViewBoxValues[2]/2,
		svgCenterY = (svgViewBoxValues[3]/2)*85/100; 

    var clicked = d3.event.target,
        direction = 1,
        factor = 0.4,
        target_zoom = 1,
        center = [svgCenterX, svgCenterY],
        extent = zoom.scaleExtent(),
        translate = zoom.translate(),
        translate0 = [],
        l = [],
        view = {x: translate[0], y: translate[1], k: zoom.scale()};

    d3.event.preventDefault();
    direction = (this.id === 'zoom_in') ? 1 : -1;
    target_zoom = zoom.scale() * (1 + factor * direction);

    if (target_zoom < extent[0] || target_zoom > extent[1]) { return false; }

    translate0 = [(center[0] - view.x) / view.k, (center[1] - view.y) / view.k];
    view.k = target_zoom;
    l = [translate0[0] * view.k + view.x, translate0[1] * view.k + view.y];

    view.x += center[0] - l[0];
    view.y += center[1] - l[1];

    interpolateZoom([view.x, view.y], view.k);
}

d3.selectAll('.zoomButton').on('click', zoomClick);

function getCentroid(element){
			bbox = element.getBBox();
			return [bbox.x + bbox.width/2, bbox.y + bbox.height/2];
		};

		function getAroundArea (element, around){
			bbox = element.getBBox();
			if (bbox.width >= bbox.height) {
				return bbox.width/2 + around;
			} else {
				return bbox.height/2 + around;
			};	
		};

function determinaZoom(object){
			return 'translate('+ object.x +','+ object.y +') scale ('+ object.scale +')'
		};

var personaggiDataset = d3.csv('_csv/_personaggi.csv', function(error, data) {
	
	if (error) throw error;

	function generateCharachtersIndex(){
		var subCSVpersonaggi = [];

		for (key in data) {
			subCSVpersonaggi.push({
				codeID: data[key].codeID,
				nome: data[key].nome
			});
		}

		subCSVpersonaggi.sort(function(a,b) {
			return d3.ascending(a.nome, b.nome);
	  	});

		d3.select('#indice_personaggi').selectAll('li').data(subCSVpersonaggi)
		.enter().append('li')
		.attr('value', function(d){
			return d.codeID;
		})
		.text(function(d){
			return d.nome;
		});
	
	}
	
	generateCharachtersIndex();

	function focusOnCharachter(transitionDuration){

		d3.select('#tempHighlight').remove();
		d3.select('#infobox').html('');
		d3.select('.firstCharIndicator').remove();

		d3.select('.activeChar').classed('activeChar', false)
		selectedChar.classed('activeChar', true)
	
		function BuildCurrentChar(obect){
			for (key in data) {
				if (selection == data[key].codeID) {
					obect.progr = data[key].progr;
					obect.codeID = data[key].codeID;
					obect.nome = data[key].nome;
					obect.luogo = data[key].luogo;
					obect.peccato = data[key].peccato;
					obect.canto = data[key].canto;
					obect.descrizione = data[key].descrizione;
					obect.wiki = data[key].wiki;
					obect.commento = data[key].commento;
					obect.x = data[key].x;
					obect.y = data[key].y;
					obect.scale = data[key].scale;
			}}
		};
	
		BuildCurrentChar(currentChar);
		
		infobox.style('display','block').append('div');
		var infoContent = infobox.select('div');

		if (currentChar.nome != '') {
			infoContent.append('h1')
			.attr('id', 'charName')
			.text(currentChar.nome)
		};

		if (currentChar.luogo != '') {
			infoContent.append('p')
			.attr('id','charPlace')
			.text(currentChar.luogo);
		};

		if (currentChar.peccato != '') {
			infoContent.append('p')
			.attr('id','charSin')
			.text(currentChar.peccato);
		};

		if (currentChar.canto != '') {
			infoContent.append('p')
			.attr('id','charCanto')
			.html(currentChar.canto);
		};

		if (currentChar.descrizione != '') {
			infoContent.append('p')
			.attr('id','charDescr')
			.text(currentChar.descrizione);

			formattedTxt = currentChar.descrizione.replace(/@/g,'<br>');

			d3.select('#charDescr').html(formattedTxt);
		};

		if (currentChar.wiki != '') {
			infoContent.append('p').attr('id','charWiki')
			.append('a')
			.attr('href',currentChar.wiki)
			.attr('target','_blank')
			.text('Wikipedia');
		};

		if (currentChar.commento != '') {
			infoContent.append('p')
			.attr('id','charComment')
			.html(currentChar.commento);
		};

		var tempCentroid = getCentroid(selectedChar.node());
		var aroundArea = getAroundArea(selectedChar.node(), 10);
		
		selectedChar.insert('circle',':first-child')
		.attr('id', 'tempHighlight')
		.attr('cx',tempCentroid[0])
		.attr('cy',tempCentroid[1])
		.attr('r', aroundArea);
		
		d3.select('#zoomContainer').call(zoom
		.translate([currentChar.x , currentChar.y])
		.scale([currentChar.scale])
		);

	 	d3.select('#zoomContainer')
		.transition().duration(transitionDuration).ease('cubic-in-out')
		.attr('transform', determinaZoom(currentChar));
	
		d3.select('#navArrows')
		.style('display', 'block');

		if (pageWidth <= breakpointMobile) {
			d3.select('#expandInfoMobile').style('visibility', 'visible');
			mobileButton.classed('open',false)
			mobileButton.classed('closed',true);
		}

	};

		if (pageWidth <= breakpointMobile) { 		
			selection = d3.select('.firstChar').attr('id'); 
			selectedChar = d3.select('#'+selection);
	 		focusOnCharachter(2000);
		};

	 	d3.selectAll('.personaggi').on('click', function(){
	 		selection = d3.select(this).attr('id');
			selectedChar = d3.select('#'+selection);
	 		focusOnCharachter(350);}
	 	);

	 	d3.selectAll('#indice_personaggi li').on('click', function(){
	 		selection = d3.select(this).attr('value');
			selectedChar = d3.select('#'+selection);

			closeMenu();
	 		focusOnCharachter(350);}
	 	);

	 	function goToNextChar(){
	 		for (var i = 0; i < data.length; i++) {
				if (selection == data[i].codeID){
					
					if (i == data.length - 1) {
						nextSelection = data[0].codeID;
					} else { nextSelection = data[i+1].codeID; }
				}
			};
			selection = nextSelection;
			selectedChar = d3.select('#'+selection);
	 		focusOnCharachter(350);
	 	};

	 	function goToPreviousChar(){
	 		for (var i = 0; i < data.length; i++) {
				if (selection == data[i].codeID){		
					if (i == 0) {
						j = data.length - 1;
						previousSelection = data[j].codeID;
					} else { previousSelection = data[i-1].codeID; }
				}
			};
			selection = previousSelection;
			selectedChar = d3.select('#'+selection);
	 		focusOnCharachter(350);
	 	};

	 	d3.select('#goRight').on('click', goToNextChar);

	 	d3.select('#goLeft').on('click', goToPreviousChar);

	 	d3.select('body').on('keydown', function(){
	 		if (pageWidth >= breakpointMobile) {
		 		if (typeof selectedChar !== 'undefined') {
		    		if (d3.event.keyCode == 39) {
		    			goToNextChar();
		    		} else if (d3.event.keyCode == 37) {
		    			goToPreviousChar();
		    		};
	    		};	
	 		};
    	});

function resetView(){

	if (pageWidth > breakpointMobile){
	d3.select('#zoomContainer').call(zoom
		.translate([0,0])
		.scale([1])
		);

	d3.select('#zoomContainer')
		.transition()
		.attr('transform', 'translate(0,0) scale(1)');

	hovering = '';
	selectedChar = '';
	selectedInfo = '';

	d3.select('#infobox').html('').style('display','none');
	d3.select('#tempHighlight').remove();
	d3.select('#navArrows').style('display','none');
	d3.selectAll('.infoActive').classed('infoActive', false);

	} else if (pageWidth <= breakpointMobile) {
		

		selection = d3.select('.firstChar').attr('id');
		selectedChar = d3.select('#'+selection);
		d3.select('#infobox').style('height',null);
		closeMenu();
 		focusOnCharachter(350);
	};
};

d3.select('#title').on('click',resetView);

function mobileMenuBehavior(CSVcolumn,clickedElement) {
	hovering = '';
	selection = '';
	selectedChar = '';

	d3.select('#infobox').style('height',null);

	mobileButton.classed('open',false);
	mobileButton.classed('closed',true);
	
	d3.select('#expandInfoMobile').classed('open',false);
	d3.select('#expandInfoMobile').classed('closed',true);
	d3.select('#expandInfoMobile').selectAll('line').transition().duration(350).ease('cubicOut').attr('transform', 'translate(0,0) rotate(0)');
	
	hovering = clickedElement.attr('value');
	
	var personaggiDataset = d3.csv('_csv/_personaggi.csv', function(error, data) {

		for (key in data){
			var passingData = data[key];
			if (hovering == passingData[CSVcolumn]) {  
				selection = passingData.codeID
				break;
				};
		};

	if (selection.length != 0) {

		selectedChar = d3.select('#'+selection);
		
		focusOnCharachter(350);
		closeMenu();
	} else {
		
		clickedElement.classed('deactivatedLink',true);
	};
	});
};

var luoghiDataset = d3.csv('_csv/_luoghi.csv', function(error, data) {
	if (error) throw error;

	var	hovering, selectedInfo;

	function showCurrentGirone(){
		hovering = d3.select(this).attr('id');
		selectedInfo = d3.select('#info_'+hovering);
		d3.selectAll('.infoActive').classed('infoActive', false);
		selectedInfo.classed('infoActive', true);
	};

	d3.selectAll('.luoghi').on('mouseenter', showCurrentGirone);
	inferno.on('click', function(){
		d3.selectAll('.luoghi').on('mouseenter', showCurrentGirone);
	});

	function setPlaceFocus(){

		d3.select('#infobox').html('');
		d3.select('#tempHighlight').remove();
		d3.select('#navArrows').style('display','none');
		d3.select('.firstCharIndicator').remove();		

		function BuildCurrentPlace(obect){
			for (key in data) {
				if (hovering == data[key].codeID) {
					obect.progr = data[key].progr;
					obect.codeID = data[key].codeID;
					obect.cerchio = data[key].cerchio;
					obect.girone = data[key].girone;
					obect.luogo = data[key].luogo;
					obect.canto = data[key].canto;
					obect.peccatori = data[key].peccatori;
					obect.punizione = data[key].punizione;
					obect.wiki = data[key].wiki;
					obect.x = data[key].x;
					obect.y = data[key].y;
					obect.scale = data[key].scale;
			}}
		};

		BuildCurrentPlace(selectedInfo);

		infobox.style('display', 'block').append('div');
		var infoContent = infobox.select('div');

		if (selectedInfo.cerchio != '') {
			infoContent.append('p')
			.attr('id', 'placeCerchio')
			.text(selectedInfo.cerchio);
		};

		if (selectedInfo.girone != '') {
			infoContent.append('p')
			.attr('id','placeGirone')
			.text(selectedInfo.girone);
		};

		if (selectedInfo.luogo != '') {
			infoContent.append('p')
			.attr('id','placeLuogo')
			.text(selectedInfo.luogo);
		};

		if (selectedInfo.canto != '') {
			infoContent.append('p')
			.attr('id','placeCanto')
			.html(selectedInfo.canto);
		};

		if (selectedInfo.peccatori != '') {
			infoContent.append('h2')
			.attr('id','peccatoriLabel')
			.text('Peccatori:');

			infoContent.append('p')
			.attr('id','placePeccatori')
			.text(selectedInfo.peccatori);	
		};
		
		if (selectedInfo.punizione != '') {
			infoContent.append('h2')
			.attr('id','punizioneLabel')
			.text('Punizione:');

			infoContent.append('p')
			.attr('id','placePunizione')
			.text(selectedInfo.punizione);
		};

		if (selectedInfo.wiki != '') {
			infoContent.append('p').attr('id','placeWiki')
			.append('a')
			.attr('href',selectedInfo.wiki)
			.attr('target','_blank')
			.text('Wikipedia');
		};
	
		d3.select('#zoomContainer').call(zoom
		.translate([selectedInfo.x , selectedInfo.y])
		.scale([selectedInfo.scale])
		);

	 	d3.select('#zoomContainer')
		.transition()
		.attr('transform', determinaZoom(selectedInfo));

		d3.selectAll('.luoghi').on('mouseenter', null)
	};

	d3.selectAll('.lateral_info').on('click', setPlaceFocus);

	d3.selectAll('.VII_cerchio, .VIII_cerchio, .IX_cerchio').on('click', function(){
		hovering = d3.select(this).attr('class');
		selectedInfo = d3.select('#info_'+hovering);
		d3.selectAll('.infoActive').classed('infoActive', false);
		selectedInfo.classed('infoActive', true);

		setPlaceFocus();
	});

	d3.select('#struttura_inferno').selectAll('a').on('click', function(){

		if (pageWidth >= breakpointMobile) { 

			hovering = d3.select(this).attr('value');
			selectedInfo = d3.select('#info_'+hovering);
			d3.selectAll('.infoActive').classed('infoActive', false);
			selectedInfo.classed('infoActive', true);
			
			closeMenu();
			setPlaceFocus();
		} else { 
			var clickedElement = d3.select(this); 
			mobileMenuBehavior('luogoID',clickedElement);
		};

	});

	d3.select('#nav_peccati').selectAll('a').on('click', function(){
		var clickedElement = d3.select(this); 
		mobileMenuBehavior('peccatoID',clickedElement);
	});

	var list = d3.select('.allSins'),
    sinSections = d3.selectAll('.sinSection'),
    subSinsLists = d3.selectAll('.subSins'),
    sins = d3.selectAll('.sin'),
    activeEl = d3.select('#active');

    function manageSections(){
        sinSections.classed('activeSect', null);;

        sins.style('font-size', '0px');

        sinSections.classed('inactiveSect', true);
        d3.select(this).classed('inactiveSect', false).classed('activeSect', true);   

        d3.selectAll('.inactiveSect').transition().duration(250).style('font-size', '0px');
        d3.selectAll('.inactiveSect').transition().delay(250).style('display', 'none');

        d3.select('.activeSect').style('display', 'block');

        d3.select(this).selectAll('.sin').style('display', 'block')
        .transition().duration(250).style('font-size', '18px');

		hovering = d3.select(this).attr('value');
		selectedInfo = d3.select('#'+hovering);
		d3.selectAll('.infoActive').classed('infoActive', false);
		selectedInfo.classed('infoActive', true);

		setPlaceFocus();
    };

    sinSections.on('click',manageSections);

    function backwardsInSinSection(){
        sins.classed('activeSin', null);
        sins.classed('inactiveSin', null);
        d3.select(this).selectAll('.sin').style('display', 'block')
        .transition().duration(250).style('font-size', '18px');
    };

    function manageSins(){
        sinSections.on('click',null);
        sins.classed('inactiveSin', true);
        d3.select(this).classed('inactiveSin', false).classed('activeSin', true);
        d3.selectAll('.inactiveSin').transition().duration(250).style('font-size', '0px');
        d3.selectAll('.inactiveSin').transition().delay(250).style('display', 'none');
        d3.selectAll('.activeSect').on('mouseup',backwardsInSinSection);    

        hovering = d3.select(this).attr('value');
		
		selectedInfo = d3.select('#'+hovering);
		d3.selectAll('.infoActive').classed('infoActive', false);
		selectedInfo.classed('infoActive', true);
		
		setPlaceFocus();  
    };

    sins.on('click', manageSins);

    function resetSinList(){
        sins.style('display', 'none');
        sins.style('font-size', '18px');
        sinSections.style('display', 'block');
        sinSections.transition().duration(250).style('font-size', '18px');

        sins.classed('activeSin', null);
        sins.classed('inactiveSin', null);
        sinSections.classed('activeSect', null);
        sinSections.classed('inactiveSect', null);

        sinSections.on('click', manageSections);
    };

    d3.select('#header').on('click',function(){
    	resetSinList();
    	resetView();
    });
}); 



var cantiDataset = d3.csv('_csv/_canti.csv', function(error, data) {
	if (error) throw error;
	function generateCantiIndex(){
		d3.select('#canti_inferno').selectAll('li').data(data)
		.enter().append('li')
		.attr('value', function(d){
			return d.codeID;
		})
		.text(function(d){
			return d.canto;
		});
	}
	
	generateCantiIndex();

	d3.selectAll('#canti_inferno li').on('click', function(){

		if (pageWidth >= breakpointMobile) { 

			hovering = d3.select(this).attr('value');
			selectedInfo = d3.selectAll('.'+hovering);
			d3.selectAll('.infoActive').classed('infoActive', false);
			selectedInfo.classed('infoActive', true);
			
			closeMenu();

			d3.select('#infobox').html('');
			d3.select('#tempHighlight').remove();
			d3.select('#navArrows').style('display','none');
			d3.select('.firstCharIndicator').remove();

			function BuildCurrentCanto(obect){
				for (key in data) {
					if (hovering == data[key].codeID) {
						obect.progr = data[key].progr;
						obect.codeID = data[key].codeID;
						obect.canto = data[key].canto;
						obect.luogo = data[key].luogo;
						obect.peccato = data[key].peccato;
						obect.personaggi = data[key].personaggi;
						obect.wiki = data[key].wiki;
						obect.x = data[key].x;
						obect.y = data[key].y;
						obect.scale = data[key].scale;
				}}
			};

			BuildCurrentCanto(selectedInfo);

			infobox.style('display', 'block').append('div');
			var infoContent = infobox.select('div');
		
			if (selectedInfo.canto != '') {
				infoContent.append('p')
				.attr('id', 'cantoNome')
				.text(selectedInfo.canto)
			};

			if (selectedInfo.luogo != '') {
				infoContent.append('p')
				.attr('id','cantoLuogo')
				.text(selectedInfo.luogo);
			};

			if (selectedInfo.peccato != '') {
				infoContent.append('p')
				.attr('id','cantoPeccato')
				.text(selectedInfo.peccato);
			};

			if (selectedInfo.personaggi != '') {
				infoContent.append('h2')
				.attr('id','personaggiLabel')
				.text('Personaggi:');

				infoContent.append('p')
				.attr('id','cantoPersonaggi')
				.text(selectedInfo.personaggi);	
			};

			if (selectedInfo.wiki != '') {
				infoContent.append('p').attr('id','cantoWiki')
				.append('a')
				.attr('href',selectedInfo.wiki)
				.attr('target','_blank')
				.text('Wikipedia');
			};
	
			d3.select('#zoomContainer').call(zoom
			.translate([selectedInfo.x , selectedInfo.y])
			.scale([selectedInfo.scale])
			);

		 	d3.select('#zoomContainer')
			.transition()
			.attr('transform', determinaZoom(selectedInfo));

			d3.selectAll('.luoghi').on('mouseenter', null);
		} else { 
			var clickedElement = d3.select(this); 
			mobileMenuBehavior('cantoID',clickedElement);
		};

	});

}); 

}); 

function closeMenu(){
		d3.selectAll('.menus').style('display','none');

		if (pageWidth <= breakpointMobile) {
			mobileButton.classed('open',false)
			mobileButton.classed('closed',true);
		};

};

d3.selectAll('.closeButton').on('click', closeMenu);
d3.select('body').on('keyup', function(){
	 		if (d3.event.keyCode == 27) {
	 			closeMenu();
	 		};
});

d3.selectAll('.aboutLink').on('click', function(){
	d3.select('#about').style('display','block');
	d3.select('#mobileNavContainer').style('display','none');
	});

d3.selectAll('.gironiLink').on('click', function(){
		d3.select('#nav_gironi').style('display','block');
		d3.select('#mobileNavContainer').style('display','none');
	});

d3.selectAll('.cantiLink').on('click', function(){
		d3.select('#nav_canti').style('display','block');
		d3.select('#mobileNavContainer').style('display','none');
});

d3.selectAll('.personaggiLink').on('click', function(){
		d3.select('#nav_personaggi').style('display','block');
		d3.select('#mobileNavContainer').style('display','none');
});

d3.selectAll('.peccatiLink').on('click', function(){
		d3.select('#nav_peccati').style('display','block');
		d3.select('#mobileNavContainer').style('display','none');
});

d3.select('#mobileButton').on('click', function(){

	if (mobileButton.attr('class')=='closed') {
		d3.select('#mobileNavContainer').style('display','block');
		mobileButton.classed('closed',false);
		mobileButton.classed('open',true);

	} else if (mobileButton.attr('class')=='open') {
		d3.select('#mobileNavContainer').style('display','none');
		mobileButton.classed('closed',true);
		mobileButton.classed('open',false);

	};
			
});

d3.select('#startNav').on('click', function(){
	var tempCentroid = getCentroid(d3.select('#dante').node());
	var aroundArea = getAroundArea(d3.select('#dante').node(), 30);

	d3.select('#dante').insert('circle',':first-child')
		.classed('firstCharIndicator', true)
		.attr('cx',tempCentroid[0])
		.attr('cy',tempCentroid[1])
		.attr('r', aroundArea);

	closeMenu();
});


d3.select('#expandInfoMobile').on('click',function(){
	var button = d3.select('button#expandInfoMobile');
	if (button.attr('class')=='closed') {
		d3.selectAll('#infobox p').style('display','block');
		d3.select('#expandInfoMobile').selectAll('line').transition().duration(350).ease('cubic-in-out').attr('transform', 'translate(25,-10) rotate(45)');
	    
	    navHeight = d3.select('#navContainer').style('height').substring(0,2);	    
		
		d3.select('#infobox').transition().duration(350).ease('cubic-out').style('height', pageHeight-navHeight+'px');
		
		button.classed('closed',false);
		button.classed('open',true);
	} else if (button.attr('class')=='open') {

		d3.selectAll('#infobox p').style('display','none');

		d3.select('#expandInfoMobile').selectAll('line').transition().duration(350).ease('cubicOut').attr('transform', 'translate(0,0) rotate(0)');
		d3.select('#infobox').transition().duration(350).ease('cubicOut').style('height', 'auto');

		button.classed('closed',true);
		button.classed('open',false);
	};	
});

d3.select('#show_instr').on('click',function(){
	d3.selectAll('#welcomePage').style('display','block');

	d3.select('#startNav').on('click', function(){
		closeMenu();
	});
});